// Copyright 2019  github.com. All rights reserved.
// Use of this source code is governed by  github.com.
package alarm

import (
	"context"
	"smart-site-management/errors"
	"smart-site-management/model"
	"smart-site-management/pb/v1"

	"github.com/jaryhe/gopkgs/database"
	"github.com/jinzhu/gorm"
	"go.uber.org/zap"
	"github.com/jaryhe/gopkgs/logger"
)

func AlarmRuleUpdate(ctx context.Context, req *v1.AlarmRuleUpdateRequest) (reply *v1.AlarmRuleUpdateReply, err error) {
	if req.Rule.Id == 0 {
		return nil, errors.ParamsError
	}
	p := model.TAlarmRule{}
	where := map[string]interface{}{
		"id": req.Rule.Id,
	}
	err = p.Query(database.DB(), where)
	if err != nil {
		if err == gorm.ErrRecordNotFound {
			return nil, errors.AlarmRuleNotExistError
		}
		logger.Error("AlarmRuleUpdate",
			zap.String("err", err.Error()))
		return nil, errors.DataBaseError
	}

	if p.ProjectId != req.Rule.ProjectId {
		return nil, errors.ProjectNotMatchError
	}
	/*
		err = utils.StructCopy(&p, req.Rule, "")
		if err != nil {
			return nil, errors.SystemError
		}

		p.Id = req.Rule.Id

		// 设置告警规则到内存
		utils.SetAlarmRule(&p)
	*/
	fileds := map[string]interface{}{
		"alarm_count":     req.Rule.AlarmCount,
		"silence_period":  req.Rule.SilencePeriod,
		"is_on":           req.Rule.IsOn,
		"continue_period": req.Rule.ContinuePeriod,
	}
	err = p.UpdateSome(database.DB(), fileds)
	if err != nil {
		logger.Error("AlarmRuleUpdate",
			zap.String("err", err.Error()))
		return nil, errors.DataBaseError
	}

	reply = &v1.AlarmRuleUpdateReply{}
	return reply, nil
}
