
package dust

import (
	"context"
	"fmt"
	"smart-site-management/errors"
	"smart-site-management/model"
	"smart-site-management/pb/v1"
	"time"
	"github.com/jaryhe/gopkgs/logger"
	"go.uber.org/zap"
)

func DustAqi(ctx context.Context, req *v1.DustAqiRequest)(reply *v1.DustAqiReply, err error) {
	layout := "2006-01-02 15:04:05"
	if req.Sn == "" {
		return nil, errors.ParamsError
	}

	dbname := getDustDbName(0)

	reply = &v1.DustAqiReply{}
	now := time.Now()
	end := now.Format(layout)
	start := now.Add(-24*time.Hour).Format(layout)

	sql := fmt.Sprintf("select MEAN(pm25) as pm25 from \"%s\" where time >= '%s' and time < '%s' tz('Asia/Shanghai')", req.Sn, start, end)
	array := []DustInfo{}
	_, err = model.QueryInfluxdb(sql, dbname, &array)
	if err != nil {
		logger.Error("DustAqi",
			zap.String("err", err.Error()))
		return nil, errors.DataBaseError
	}

	if len(array) == 0 {
		reply.Aqi = 0
		return reply, nil
	}
	pm25 := array[0].Pm25Avg
	switch {
	case pm25 >= 0 && pm25 < 50:
		reply.Aqi = pm25
	case pm25 >= 50 && pm25 < 150:
		reply.Aqi = (50.0/100.0) *(pm25 - 50.0) + 50
	case pm25 >= 150 && pm25 < 250:
		reply.Aqi = (50.0/100.0) *(pm25 - 150.0) + 100
	case pm25 >= 250 && pm25 < 350:
		reply.Aqi = (50.0/100.0) *(pm25 - 250.0) + 150
	case pm25 >= 350 && pm25 < 420:
		reply.Aqi = (100.0/70.0) *(pm25 - 350.0) + 200
	case pm25 >= 420 && pm25 < 500:
		reply.Aqi = (100.0/80.0) *(pm25 - 420.0) + 300
	case pm25 >= 500 && pm25 < 600:
		reply.Aqi =  (pm25 - 500.0) + 400
	}
	return reply, nil
}

