package dust

import (
	"context"
	"fmt"
	"github.com/jaryhe/gopkgs/logger"
	"go.uber.org/zap"
	"smart-site-management/errors"
	"smart-site-management/model"
	"smart-site-management/pb/v1"
	"strings"
	"time"
)

// 获取分钟 06:10 06:11
func getMinute(t string) string {
	tmp := strings.Split(t, " ")
	if len(tmp) < 2 {
		return t
	}

	array := strings.Split(tmp[1], ":")
	if len(array) < 3 {
		return t
	}
	return fmt.Sprintf("%s:%s", array[0], array[1])
}

// 获取小时 06:00 07:00
func getHour(t string) string {
	tmp := strings.Split(t, " ")
	if len(tmp) < 2 {
		return t
	}
	array := strings.Split(tmp[1], ":")
	if len(array) < 3 {
		return t
	}
	return fmt.Sprintf("%s:%s", array[0], array[1])
}

// 获取天 1号 2号
func getDay(t string) string {
	tmp := strings.Split(t, " ")
	if len(tmp) < 2 {
		return t
	}
	array := strings.Split(tmp[0], "-")
	if len(array) < 3 {
		return t
	}
	day := array[2]
	if day[0] == '0' {
		day = string(day[1])
	}
	return fmt.Sprintf("%s号", day)
}

// 根据类型获取时间
func getTimeByType(t string, reqType int32) string {
	switch reqType {
	case 0:
		return getMinute(t)
	case 1:
		return getHour(t)
	case 2:
		return getDay(t)
	}
	return getMinute(t)
}

func getTimeArray(reqTime int64, reqType int32) []string {
	layout := "2006-01-02 15:04:05"
	start := time.Unix(reqTime, 0)
	ret := []string{}

	switch reqType {
	case 0:
		startHour := start.Hour()
		for {
			if start.Hour() != startHour {
				break
			}
			minute := getMinute(start.Format(layout))
			ret = append(ret, minute)
			start = start.Add(1 * time.Minute)
		}
	case 1:
		startDay := start.Day()
		for {
			if start.Day() != startDay {
				break
			}
			hour := getHour(start.Format(layout))
			ret = append(ret, hour)
			start = start.Add(60 * time.Minute)
		}
	case 2:
		startMonth := start.Month()
		for {
			if start.Month() != startMonth {
				break
			}
			day := getDay(start.Format(layout))
			ret = append(ret, day)
			start = start.Add(24 * 60 * time.Minute)
		}
	}
	return ret
}

func getStartAndEnd(reqTime int64, reqType int32) (string, string) {
	layout := "2006-01-02 15:04:05"
	start := time.Unix(reqTime, 0)
	end := start
	switch reqType {
	case 0:
		end = start.Add(60 * time.Minute)
	case 1:
		end = start.Add(24 * 60 * time.Minute)
	case 2:
		for {
			end = end.Add(24 * 60 * time.Minute)
			if end.Month() != start.Month() {
				break
			}
		}
	default:
		end = start.Add(60 * time.Minute)
	}
	return start.Format(layout), end.Format(layout)
}

func tabExist(sn string, dbname string) bool {
	sql := fmt.Sprintf("select * from \"%s\" limit 10", sn)
	array := []DustInfo{}
	model.QueryInfluxdb(sql, dbname, &array)
	if len(array) == 0 {
		return false
	}
	return true
}

// 当小时表和天表无数据时从分钟表里统计
func getDustHistoryFromMinuteSql(req *v1.DustMonitorHistoryRequest) (string, string) {
	group := "1h"
	if req.Type == 2 {
		group = "1d"
	}
	sql := fmt.Sprintf("select mean(*) from \"%s\" ", req.Sn)
	where := ""
	if req.Time > 0 {
		start, end := getStartAndEnd(req.Time, req.Type)
		where = fmt.Sprintf("where time >= '%s' and time < '%s'",
			start,
			end)
	}
	//countSql := fmt.Sprintf("select count(mn) as count from \"%s\" %s tz('Asia/Shanghai')", req.Sn, where)
	sql = fmt.Sprintf("%s %s group by time(%s) order by time tz('Asia/Shanghai')", sql, where, group)
	fmt.Printf("sql:%s\n", sql)
	return sql, ""
}

func getDustHistorySql(req *v1.DustMonitorHistoryRequest) (string, string, string) {
	dbname := getDustDbName(req.Type)
	if req.Type != 0 && !tabExist(req.Sn, dbname) {
		sql, countSql := getDustHistoryFromMinuteSql(req)
		return sql, countSql, getDustDbName(0)
	}
	//layout := "2006-01-02 15:04:05"
	sql := fmt.Sprintf("select * from \"%s\" ", req.Sn)
	where := ""
	if req.Time > 0 {
		start, end := getStartAndEnd(req.Time, req.Type)
		where = fmt.Sprintf("where time >= '%s' and time < '%s'",
			start,
			end)
	}
	//countSql := fmt.Sprintf("select count(mn) as count from \"%s\" %s tz('Asia/Shanghai')", req.Sn, where)
	sql = fmt.Sprintf("%s %s order by time tz('Asia/Shanghai')", sql, where)
	fmt.Printf("sql:%s\n", sql)
	return sql, "", dbname
}

func getDustHistoryCount(dbname string, sql string) (int64, error) {
	type Result struct {
		Count int64
	}
	array := []Result{}
	_, err := model.QueryInfluxdb(sql, dbname, &array)
	if err != nil {
		return 0, errors.DataBaseError
	}
	if len(array) == 0 {
		return 0, nil
	}
	return array[0].Count, nil
}

// 从分钟表统计小时数据或天数据
func dustMonitorHistoryByGroup(req *v1.DustMonitorHistoryRequest, sql, dbname string) (reply *v1.DustMonitorHistoryReply, err error) {
	array := []DustInfoMean{}
	reply = &v1.DustMonitorHistoryReply{}
	_, err = model.QueryInfluxdb(sql, dbname, &array)
	if err != nil {
		logger.Error("DustMonitorHistory",
			zap.String("err", err.Error()))
		return nil, errors.DataBaseError
	}

	m := map[string][]v1.DustMonitorData{}

	// 获取时段坐标
	tarray := getTimeArray(req.Time, req.Type)
	for _, v := range tarray {
		m[v] = []v1.DustMonitorData{}
	}

	for i, v := range array {
		item := v1.DustMonitorData{}
		item.Date = getTimeByType(v.Time, req.Type)
		item.B03 = array[i].B03Avg
		item.H01 = array[i].H01Avg
		item.P01 = array[i].P01Avg
		item.Pm10 = array[i].Pm10Avg
		item.Pm25 = array[i].Pm25Avg
		item.R01 = array[i].R01Avg
		item.T01 = array[i].T01Avg
		item.Tsp = array[i].TspAvg
		item.W01 = array[i].W01Avg
		item.W02 = array[i].W02Avg
		m[item.Date] = append(m[item.Date], item)

	}

	// 一个时段如果有多条数据算平均值
	reply.List = make([]*v1.DustMonitorData, len(tarray))
	for i, k := range tarray {
		subarray := m[k]
		if len(subarray) == 0 {
			reply.List[i] = &v1.DustMonitorData{Date: k}

			continue
		}
		newItem := v1.DustMonitorData{}
		length := len(subarray)
		for _, v := range subarray {
			newItem.Date = v.Date
			newItem.B03 += v.B03
			newItem.H01 += v.H01
			newItem.P01 += v.P01
			newItem.Pm10 += v.Pm10
			newItem.Pm25 += v.Pm25
			newItem.R01 += v.R01
			newItem.T01 += v.T01
			newItem.Tsp += v.Tsp
			newItem.W01 += v.W01
			newItem.W02 += v.W02
		}
		newItem.B03 = newItem.B03 / float64(length)
		newItem.H01 = newItem.H01 / float64(length)
		newItem.P01 = newItem.P01 / float64(length)
		newItem.Pm10 = newItem.Pm10 / float64(length)
		newItem.Pm25 = newItem.Pm25 / float64(length)
		newItem.R01 = newItem.R01 / float64(length)
		newItem.T01 = newItem.T01 / float64(length)
		newItem.Tsp = newItem.Tsp / float64(length)
		newItem.W01 = newItem.W01 / float64(length)
		newItem.W02 = newItem.W02 / float64(length)
		reply.List[i] = &newItem
	}

	return reply, nil
}

func DustMonitorHistory(ctx context.Context, req *v1.DustMonitorHistoryRequest) (reply *v1.DustMonitorHistoryReply, err error) {
	if req.Time == 0 || req.Sn == "" {
		return nil, errors.ParamsError
	}

	reply = &v1.DustMonitorHistoryReply{}

	sql, _, dbname := getDustHistorySql(req)
	if err != nil {
		logger.Error("DustMonitorHistory",
			zap.String("err", err.Error()))
		return nil, err
	}


	if dbname != getDustDbName(req.Type) {
		return dustMonitorHistoryByGroup(req, sql, dbname)
	}
	array := []DustInfo{}
	_, err = model.QueryInfluxdb(sql, dbname, &array)
	if err != nil {
		logger.Error("DustMonitorHistory",
			zap.String("err", err.Error()))
		return nil, errors.DataBaseError
	}

	m := map[string][]v1.DustMonitorData{}

	// 获取时段坐标
	tarray := getTimeArray(req.Time, req.Type)
	for _, v := range tarray {
		m[v] = []v1.DustMonitorData{}
	}

	for i, v := range array {
		item := v1.DustMonitorData{}
		item.Date = getTimeByType(v.Time, req.Type)
		item.B03 = array[i].B03Avg
		item.H01 = array[i].H01Avg
		item.P01 = array[i].P01Avg
		item.Pm10 = array[i].Pm10Avg
		item.Pm25 = array[i].Pm25Avg
		item.R01 = array[i].R01Avg
		item.T01 = array[i].T01Avg
		item.Tsp = array[i].TspAvg
		item.W01 = array[i].W01Avg
		item.W02 = array[i].W02Avg
		m[item.Date] = append(m[item.Date], item)

	}

	// 一个时段如果有多条数据算平均值
	reply.List = make([]*v1.DustMonitorData, len(tarray))
	for i, k := range tarray {
		subarray := m[k]
		if len(subarray) == 0 {
			reply.List[i] = &v1.DustMonitorData{Date: k}

			continue
		}
		newItem := v1.DustMonitorData{}
		length := len(subarray)
		for _, v := range subarray {
			newItem.Date = v.Date
			newItem.B03 += v.B03
			newItem.H01 += v.H01
			newItem.P01 += v.P01
			newItem.Pm10 += v.Pm10
			newItem.Pm25 += v.Pm25
			newItem.R01 += v.R01
			newItem.T01 += v.T01
			newItem.Tsp += v.Tsp
			newItem.W01 += v.W01
			newItem.W02 += v.W02
		}
		newItem.B03 = newItem.B03 / float64(length)
		newItem.H01 = newItem.H01 / float64(length)
		newItem.P01 = newItem.P01 / float64(length)
		newItem.Pm10 = newItem.Pm10 / float64(length)
		newItem.Pm25 = newItem.Pm25 / float64(length)
		newItem.R01 = newItem.R01 / float64(length)
		newItem.T01 = newItem.T01 / float64(length)
		newItem.Tsp = newItem.Tsp / float64(length)
		newItem.W01 = newItem.W01 / float64(length)
		newItem.W02 = newItem.W02 / float64(length)
		reply.List[i] = &newItem
	}

	return reply, nil
}
