// Copyright 2019  githup.com. All rights reserved.
// Use of this source code is governed by  githup.com.
package project

import (
	"context"
	"encoding/base64"
	"fmt"
	"github.com/jaryhe/gopkgs/database"
	"github.com/jaryhe/gopkgs/logger"
	"github.com/jaryhe/gopkgs/util"
	"go.uber.org/zap"
	"smart-site-management/consts"
	"smart-site-management/errors"
	dbmodel "smart-site-management/model"
	pb_v1 "smart-site-management/pb/v1"
	"time"
)

const (
	// 0 供应商入驻审批 1 设备对接审批  3 拆机审批
	ApproveTypeProvider = iota
	ApproveTypeDeviceDocking
	ApproveTypeDeviceDel
)


const CRYPTO_KEY = "2D29BP43U3Y1B4N6REQW3F319DD23454"

func makeDeviceKey(sn string, deviceType int32 ) (string) {
	cryPasswd, _ := util.AesEncrypt(fmt.Sprintf("%s%d", sn, deviceType), CRYPTO_KEY)
	ret := base64.StdEncoding.EncodeToString(cryPasswd)
	return ret
}

func DeviceApprove(ctx context.Context, req *pb_v1.DeviceApproveRequest)(reply *pb_v1.DeviceApproveReply, err error) {
	p := &dbmodel.TDevice{}
	where := map[string]interface{}{
		"id":req.Id,
	}
	// 状态判断
	err = p.Find(database.DB(), where)
	if err != nil {
		logger.Error("DeviceApprove",
			zap.String("err", err.Error()))
		return nil, errors.DataBaseError
	}
	if p.Id == 0 {
		return nil, errors.NoRecordError
	}
	if p.VerifyStatus != consts.DeviceStatusAddAuditting {
		return nil, errors.CanNotApproveWithStatus
	}

	// 更新
	where = map[string]interface{}{
		"id":req.Id,
		"verify_status":consts.DeviceStatusAddAuditting,
	}
	value := map[string]interface{}{
		"verify_status":consts.DeviceStatusAddAuditted,
		"project_approve_time":time.Now(),
		"key": makeDeviceKey(p.Sn, p.DeviceCode),

	}
	if !req.Status {
		value["verify_status"] = consts.DeviceStatusAddAudittedFail
	}
	err = p.Update(database.DB(), where, value)
	if err != nil {
		logger.Error("DeviceApprove",
			zap.String("err", err.Error()))
		return nil, errors.DataBaseError
	}
	return &pb_v1.DeviceApproveReply{}, nil
}
