// Copyright 2019  github.com. All rights reserved.
// Use of this source code is governed by  github.com.
package staff

import (
	"context"
	"github.com/jaryhe/gopkgs/database"
	"github.com/jaryhe/gopkgs/logger"
	"go.uber.org/zap"
	"smart-site-management/errors"
	"smart-site-management/model"
	"smart-site-management/pb/v1"
)

func StaffTypeStatistics(ctx context.Context, req *v1.StaffTypeStatisticsRequest) (reply *v1.StaffTypeStatisticsReply, err error) {
	reply = &v1.StaffTypeStatisticsReply{}
	if req.ProjectId == 0 {
		return nil,errors.ParamsError
	}

	list, err := model.StaffTypeStaticList(database.DB(), req.ProjectId)
	if err != nil {
		logger.Error("StaffTypeStaticList",
			zap.String("err", err.Error()))
		return nil, errors.DataBaseError
	}

	reply.List = make([]*v1.StaffTypeStatistics, len(list))
	totalAttendance := 0
	totalIn := 0
	for i, v := range list{
		item := &v1.StaffTypeStatistics{
			StaffType:v.TypeName,
			Total:int64(v.StaffCount),
			Attendance:int64(v.StaffAttendanceCount),
			TypeCode:v.TypeCode,
		}
		totalAttendance += v.StaffAttendanceCount
		totalIn += v.StaffInCount
		reply.List[i] = item
	}
	reply.TotalAttendance = int64(totalAttendance)
	reply.TotalIn = int64(totalIn)
	return reply, nil
}
