package tower

import (
	"context"
	"fmt"
	"github.com/jaryhe/gopkgs/logger"
	"go.uber.org/zap"
	"smart-site-management/errors"
	"smart-site-management/model"
	"smart-site-management/pb/v1"
	"time"
)


func getTowerHistorySql(req *v1.TowerMonitorHistoryRequest) (string, string, string) {
	//layout := "2006-01-02 15:04:05"
	sql := fmt.Sprintf("select * from \"%s\" ", req.Sn)
	where := ""
	whereArray := []string{}
	if req.Start > 0 {
		whereArray = append(whereArray, fmt.Sprintf("time >= '%s'", time.Unix(req.Start, 0).Format("2006-01-02 15:04:05")))
	}
	if req.End > 0 {
		whereArray = append(whereArray, fmt.Sprintf("time < '%s'", time.Unix(req.End, 0).Format("2006-01-02 15:04:05")))
	}
	for _, v := range whereArray {
		if where == "" {
			where = fmt.Sprintf("where %s", v)
			continue
		}
		where = fmt.Sprintf("%s and %s", where, v)
	}

	offset := int64(req.Page - 1) * int64(req.PageSize)
	countSql := fmt.Sprintf("select count(back_turn) as count from \"%s\" %s tz('Asia/Shanghai')", req.Sn, where)
	sql = fmt.Sprintf("%s %s order by time limit %d offset %d tz('Asia/Shanghai')", sql, where, req.PageSize, offset)
	fmt.Printf("sql:%s\n", sql)
	return sql, countSql, "tower_monitor_2"
}

func getTowerHistoryCount(dbname string, sql string) (int64, error) {
	type Result struct {
		Count int64
	}
	array := []Result{}
	_, err := model.QueryInfluxdb(sql, dbname, &array)
	if err != nil {
		return 0, errors.DataBaseError
	}
	if len(array) == 0 {
		return 0, nil
	}
	return array[0].Count, nil
}


func TowerMonitorHistory(ctx context.Context, req *v1.TowerMonitorHistoryRequest) (reply *v1.TowerMonitorHistoryReply, err error) {
	if req.Sn == "" {
		return nil, errors.ParamsError
	}
	if req.PageSize == 0 {
		req.PageSize = 10
	}
	if req.Page == 0 {
		req.Page = 1
	}


	reply = &v1.TowerMonitorHistoryReply{
		Page:req.Page,
		PageSize:req.PageSize,
	}

	sql, countSql, dbname := getTowerHistorySql(req)
	if err != nil {
		logger.Error("DustMonitorHistory",
			zap.String("err", err.Error()))
		return nil, err
	}
	total, err := getTowerHistoryCount(dbname, countSql)
	if err != nil {
		logger.Error("DustMonitorHistory",
			zap.String("err", err.Error()))
		return nil, err
	}
	reply.Total = total
	if reply.Total == 0 {
		return reply, nil
	}

	array := []TowerInfo{}
	_, err = model.QueryInfluxdb(sql, dbname, &array)
	if err != nil {
		logger.Error("TowerMonitorHistory",
			zap.String("err", err.Error()))
		return nil, errors.DataBaseError
	}


	for i, v := range array {
		item := v1.TowerMonitorData{}
		item.Date = v.Time
		item.AngleY = array[i].AngleY
		item.AngleX = array[i].AngleX
		item.WindSpeed = array[i].WindSpeed
		item.Battery = fmt.Sprintf("%d", array[i].Battery)+"%"
		item.Weight = array[i].Weight
		item.Hight = array[i].Hight
		item.Scope = array[i].Scope
		item.BackTurn = array[i].BackTurn
		item.Moment = fmt.Sprintf("%d", array[i].Moment)+"%"
		reply.List = append(reply.List, &item)
	}

	return reply, nil
}
