// Copyright 2019  github.com. All rights reserved.
// Use of this source code is governed by  github.com.
package vehicle

import (
	"context"
	"fmt"
	"smart-site-management/model"
	"smart-site-management/pb/v1"
	"smart-site-management/errors"
	"github.com/jaryhe/gopkgs/logger"
	"go.uber.org/zap"
)

type VehicleInfo struct {
	PlateNo string `json:"plate_no"`
	Type int `json:"type"`
	Image string `json:"image"`
	Sn string `json:"sn"`
	CleanImage string `json:"clean_image"`
	VehicleType int32 `json:"vehicle_type"`
	Time string `json:"time"`
}

func VehicleList(ctx context.Context, req *v1.VehicleListRequest) (reply *v1.VehicleListReply, err error) {
	if req.ProjectId == 0 {
		return nil, errors.ParamsError
	}
	if req.PageSize == 0 {
		req.PageSize = 10
	}
	if req.Page == 0 {
		req.Page = 1
	}
	reply = &v1.VehicleListReply{
		Page:req.Page,
		PageSize:int32(req.PageSize),
	}
	offset := int((req.Page - 1)) *int(req.PageSize)

	whereArray := []string{}
	if req.Type > 0 {
		whereArray = append(whereArray, fmt.Sprintf("type=%d", req.Type))
	}
	if req.Start > 0 {
		whereArray = append(whereArray, fmt.Sprintf("time >= %ds", req.Start))
	}
	if req.End > 0 {
		whereArray = append(whereArray, fmt.Sprintf("time < %ds", req.End))
	}
	if req.PlateNo != "" {
		whereArray = append(whereArray, fmt.Sprintf("plate_no=~/%s/", req.PlateNo))
	}
	where := ""
	for _, v := range whereArray {
		if where == "" {
			where = fmt.Sprintf("where %s", v)
		} else {
			where = fmt.Sprintf("%s %s", where, v)
		}
	}
	// prepare sql
	sql :=  fmt.Sprintf("select * from \"project_%d\"  %s order by time desc limit %d offset %d tz('Asia/Shanghai')",req.ProjectId,  where, req.PageSize, offset)
	countSql := fmt.Sprintf("select count(type) as count from \"project_%d\" %s tz('Asia/Shanghai')",req.ProjectId, where)

	// get count
	type Result struct {
		Count int64
	}
	array := []Result{}
	_, err = model.QueryInfluxdb(countSql, "vehicle", &array)
	if err != nil {
		logger.Error("VehicleList",
			zap.String("err", err.Error()))
		return nil, errors.DataBaseError
	}
	if len(array) == 0 {
		return  reply, nil
	}
	reply.Total = array[0].Count

	// get list
	lArray := []VehicleInfo{}
	_, err = model.QueryInfluxdb(sql, "vehicle", &lArray)

	if err != nil {
		logger.Error("VehicleList",
			zap.String("err", err.Error()))
		return nil, errors.DataBaseError
	}
	if len(lArray) == 0 {
		return  reply, nil
	}

	reply.List = make([]*v1.Vehicle, len(lArray))
	for i, v := range lArray {
		item := &v1.Vehicle{}
		item.Id = int64(offset+i+1)
		item.PlateNo = v.PlateNo
		item.CleanImage = v.CleanImage
		item.Image = v.Image
		item.Date = v.Time
		item.Sn = v.Sn
		item.Type = "出"
		if v.Type == 1 {
			item.Type = "进"
		}
		if v.VehicleType == 1 {
			item.VehicleType = "运渣车"
		} else if v.VehicleType == 0 {
			item.VehicleType = "轿车"
		} else if v.VehicleType == 2 {
			item.VehicleType = "罐车"
		} else {
			item.VehicleType = "其他"
		}
		reply.List[i] = item
	}
	return reply, nil
}
