// Copyright 2019  github.com. All rights reserved.
// Use of this source code is governed by  github.com.
package model

import (
	"github.com/jinzhu/gorm"
	"go.uber.org/zap"
	"smart-site-management/errors"
	"time"
	"github.com/jaryhe/gopkgs/logger"
	"encoding/json"
)

type TProject struct {
	Id int64 `gorm:"column:id" json:"id" form:"id" `
	Cid int64 `gorm:"column:cid" json:"cid" form:"cid"`
	SafetyRecordNo string `gorm:"column:safety_record_no" json:"safety_record_no" form:"safety_record_no"`
	ProjectNo string `gorm:"column:project_no" json:"project_no" form:"project_no"`
	Name string `gorm:"column:name" json:"name" form:"name"`
	ShortName string `gorm:"column:short_name" json:"short_name" form:"short_name"`
	ProjectFunction string `gorm:"column:project_function" json:"project_function" form:"project_function"`
	ProjectType string `gorm:"column:project_type" json:"project_type" form:"project_type"`
	Price float64 `gorm:"column:price" json:"price" form:"price"`
	Desc string `gorm:"column:desc" json:"desc" form:"desc"`
	ProjectLeader string `gorm:"column:project_leader" json:"project_leader" form:"project_leader"`
	CoveredArea float64 `gorm:"column:covered_area" json:"covered_area" form:"covered_area"`
	Phone string `gorm:"column:phone" json:"phone" form:"phone"`
	TotalLevel int64 `gorm:"column:total_level" json:"total_level" form:"total_level"`
	BuildingStorey string `gorm:"column:building_storey" json:"building_storey" form:"building_storey"`
	Construction string `gorm:"column:construction" json:"construction" form:"construction"`
	Development string `gorm:"column:development" json:"development" form:"development"`
	Oversee string `gorm:"column:oversee" json:"oversee" form:"oversee"`
	Design string `gorm:"column:design" json:"design" form:"design"`
	Exploration string `gorm:"column:exploration" json:"exploration" form:"exploration"`
	Location string `gorm:"column:location" json:"location" form:"location"`
	Lon float64 `gorm:"column:lon" json:"lon" form:"lon"`
	Lat float64 `gorm:"column:lat" json:"lat" form:"lat"`
	ProjectPlan string
	ProjectEffectPic string
	StartDay time.Time
	EndDay time.Time
	IsFinish int64 `gorm:"column:is_finish" json:"is_finish" form:"is_finish"`
	CreatedAt time.Time `gorm:"column:created_at" json:"created_at" form:"created_at"`
	UpdatedAt time.Time `gorm:"column:updated_at" json:"updated_at" form:"updated_at"`
}

func (TProject) TableName() string {
	return "t_project"
}

func (p *TProject) List(db *gorm.DB, where map[string]interface{}, or map[string]interface{}, page int32) (list []TProject, err error) {
	offset := (page - 1) * int32(PageSize)
	if len(where) > 0 || len(or) > 0 {
		cond, val, err := whereBuildAndOr(where, or)
		if err != nil {
			return list, err
		}

		err = db.Table(p.TableName()).Where(cond, val...).Limit(PageSize).Offset(offset).Find(&list).Error
		if err != nil {
			fields, _ := json.Marshal(where)
			logger.Error("mysql",
				zap.String("sql", "select * from t_project"),
				zap.String("fields", string(fields)),
				zap.String("error", err.Error()))
			return  list, errors.DataBaseError
		}

		return list, err
	}
	err = db.Table(p.TableName()).Find(&list).Error
	if err != nil {
		logger.Error("mysql",
			zap.String("sql", "select * from t_project"),
			zap.String("error", err.Error()))
		return  list, errors.DataBaseError
	}

	return list, nil
}


func (p *TProject) Insert(db *gorm.DB) error {
	err := db.Create(p).Error
	if err != nil {
		fields, _ := json.Marshal(p)
		logger.Error("mysql",
			zap.String("sql", "insert into t_project"),
			zap.String("fields", string(fields)),
			zap.String("error", err.Error()))
		return  errors.DataBaseError
	}
	return err
}

func (p *TProject) Del(db *gorm.DB, where map[string]interface{}) error {
	cond, val, err := whereBuild(where)
	if err != nil {
		return err
	}
	err =  db.Table(p.TableName()).Where(cond, val...).Delete(p).Error
	if err != nil {
		fields, _ := json.Marshal(where)
		logger.Error("mysql",
			zap.String("sql", "delete from t_project"),
			zap.String("fields", string(fields)),
			zap.String("error", err.Error()))
		return  errors.DataBaseError
	}
	return err
}

func (p *TProject) Find(db *gorm.DB, where map[string]interface{}) error {
	cond, val, err := whereBuild(where)
	if err != nil {
		return err
	}
	err = db.Table(p.TableName()).Where(cond, val...).First(p).Error
	if err != nil {
		fields, _ := json.Marshal(where)
		logger.Error("mysql",
			zap.String("sql", "select from t_project"),
			zap.String("fields", string(fields)),
			zap.String("error", err.Error()))
		if err == gorm.ErrRecordNotFound {
			return nil
		}
		return  errors.DataBaseError
	}
	return err
}

func (p *TProject) Update(db *gorm.DB, where map[string]interface{}, values map[string]interface{}) error {
	cond, val, err := whereBuild(where)
	if err != nil {
		return err
	}

	err = db.Table(p.TableName()).Where(cond, val...).Updates(values).Error
	if err != nil {
		fields, _ := json.Marshal(where)
		updateValues, _ := json.Marshal(values)
		logger.Error("mysql",
			zap.String("sql", "update t_project"),
			zap.String("fields", string(fields)),
			zap.String("values", string(updateValues)),
			zap.String("error", err.Error()))
		return  errors.DataBaseError
	}
	return err
}


func (p *TProject) Count(db *gorm.DB, where map[string]interface{}, or map[string]interface{}) (int64, error) {
	if len(where) > 0 || len(or) > 0{
		cond, val, err := whereBuildAndOr(where, or)
		if err != nil {
			return 0, err
		}
		ret := int64(0)
		err = db.Table(p.TableName()).Where(cond, val...).Count(&ret).Error
		if err != nil {
			fields, _ := json.Marshal(val)
			logger.Error("mysql",
				zap.String("sql", "select count(1) t_project"),
				zap.String("fields", cond + ","+string(fields)),
				zap.String("error", err.Error()))
			return  ret, errors.DataBaseError
		}

		return ret, err
	}
	ret := int64(0)
	err := db.Table(p.TableName()).Count(&ret).Error
	if err != nil {
		logger.Error("mysql",
			zap.String("sql", "select count(1) t_project"),
			zap.String("error", err.Error()))
		return  ret, errors.DataBaseError
	}
	return ret, err
}


