// Copyright 2019  github.com. All rights reserved.
// Use of this source code is governed by  github.com.
package model

import (
	"github.com/jinzhu/gorm"
	"go.uber.org/zap"
	"smart-site-management/errors"
	"time"
	"github.com/jaryhe/gopkgs/logger"
	"encoding/json"
)

const (
	ProjectJobTypeDelDevice = 1
	ProjectJobTypeUpdateProject = 2
)
type TProjectJob struct {
	ID int64 `gorm:"column:id" json:"id" form:"id"`
	Type int64 `gorm:"column:type" json:"type" form:"type"`
	Origin string `gorm:"column:origin" json:"origin" form:"origin"`
	Content string `gorm:"column:content" json:"content" form:"content"`
	CreatedAt time.Time `gorm:"column:created_at" json:"created_at" form:"created_at"`
	Status int32 `gorm:"column:status" json:"status" form:"status"`
	UpdatedAt time.Time `gorm:"column:updated_at" json:"updated_at" form:"updated_at"`
	ProjectId int64 `gorm:"column:project_id" json:"project_id" form:"project_id"`
	Feedback string `json:"feedback"`
	Reason string `json:"reason"`
	ProviderId int64  `json:"provider_id"`
	DeviceId int64 `json:"device_id"`
}

func (TProjectJob) TableName() string {
	return "t_project_job"
}

func (p *TProjectJob) Insert(db *gorm.DB) error {
	err := db.Create(p).Error
	if err != nil {
		fields, _ := json.Marshal(p)
		logger.Error("mysql",
			zap.String("sql", "insert into t_project_job"),
			zap.String("fields", string(fields)),
			zap.String("error", err.Error()))
		return  errors.DataBaseError
	}
	return err
}


func (p *TProjectJob) Find(db *gorm.DB, where map[string]interface{}) error {
	cond, val, err := whereBuild(where)
	if err != nil {
		return err
	}
	err = db.Table(p.TableName()).Where(cond, val...).First(p).Error
	if err != nil {
		fields, _ := json.Marshal(where)
		logger.Error("mysql",
			zap.String("sql", "select from t_project_job"),
			zap.String("fields", string(fields)),
			zap.String("error", err.Error()))
		return  errors.DataBaseError
	}
	return err
}

func (p *TProjectJob) Update(db *gorm.DB, where map[string]interface{}, values map[string]interface{}) error {
	cond, val, err := whereBuild(where)
	if err != nil {
		return err
	}

	err =  db.Table(p.TableName()).Where(cond, val...).Updates(values).Error
	if err != nil {
		fields, _ := json.Marshal(where)
		updateValues, _ := json.Marshal(values)
		logger.Error("mysql",
			zap.String("sql", "update t_project_job"),
			zap.String("fields", string(fields)),
			zap.String("values", string(updateValues)),
			zap.String("error", err.Error()))
		return  errors.DataBaseError
	}
	return err
}

func (p *TProjectJob) Count(db *gorm.DB, where map[string]interface{}) (int64, error) {
	if len(where) > 0 {
		cond, val, err := whereBuild(where)
		if err != nil {
			return 0, err
		}
		ret := int64(0)
		err = db.Table(p.TableName()).Where(cond, val...).Count(&ret).Error
		if err != nil {
			fields, _ := json.Marshal(where)
			logger.Error("mysql",
				zap.String("sql", "select count(1) from t_project_job"),
				zap.String("fields", string(fields)),
				zap.String("error", err.Error()))
			return  ret, errors.DataBaseError
		}
		return ret, err
	}
	ret := int64(0)
	err := db.Table(p.TableName()).Count(&ret).Error
	if err != nil {
		fields, _ := json.Marshal(where)
		logger.Error("mysql",
			zap.String("sql", "select count(1) from t_project_job"),
			zap.String("fields", string(fields)),
			zap.String("error", err.Error()))
		return  ret, errors.DataBaseError
	}
	return ret, err
}

func (p *TProjectJob) List(db *gorm.DB, where map[string]interface{}, page int) (list []TProjectJob, err error) {
	offset := (page - 1) *PageSize
	if len(where) > 0 {
		cond, val, err := whereBuild(where)
		if err != nil {
			return list, err
		}

		err = db.Table(p.TableName()).Where(cond, val...).Limit(PageSize).Offset(offset).Find(&list).Error
		if err != nil {
			fields, _ := json.Marshal(where)
			logger.Error("mysql",
				zap.String("sql", "select count * from t_project_job"),
				zap.String("fields", string(fields)),
				zap.String("error", err.Error()))
			return  list, errors.DataBaseError
		}
		return list, err
	}
	err = db.Table(p.TableName()).Limit(10).Offset(page).Find(&list).Error
	if err != nil {
		logger.Error("mysql",
			zap.String("sql", "select count * from t_project_job"),
			zap.String("error", err.Error()))
		return  list, errors.DataBaseError
	}

	return list, err

}

