// Copyright 2019  githup.com. All rights reserved.
// Use of this source code is governed by  githup.com.
package model


import (
	"github.com/jinzhu/gorm"
	"go.uber.org/zap"
	"smart-site-management/errors"
	"time"
	"github.com/jaryhe/gopkgs/logger"
	"encoding/json"

)

type TProviderDevice struct {
	Id int64 `gorm:"column:id" json:"id" form:"id"`
	ProviderId int64 `gorm:"column:provider_id" json:"provider_id" form:"provider_id"`
	DeviceCode int32 `gorm:"column:device_code" json:"device_code" form:"device_code"`
	Status int32 `gorm:"column:status" json:"status" form:"status"`
	CreatedAt time.Time
	UpdatedAt time.Time
}

func (TProviderDevice) TableName() string {
	return "t_provider_device"
}

func (p *TProviderDevice) Insert(db *gorm.DB) error {
	err := db.Create(p).Error
	if err != nil {
		fields, _ := json.Marshal(p)
		logger.Error("mysql",
			zap.String("sql", "insert into t_provider_device"),
			zap.String("fields", string(fields)),
			zap.String("error", err.Error()))
		return  errors.DataBaseError
	}
	return err
}

func (p *TProviderDevice) Find(db *gorm.DB, where map[string]interface{}) error {
	cond, val, err := whereBuild(where)
	if err != nil {
		return err
	}
	err =  db.Table(p.TableName()).Where(cond, val...).First(p).Error
	if err != nil {
		fields, _ := json.Marshal(where)
		logger.Error("mysql",
			zap.String("sql", "select from t_provider_device"),
			zap.String("fields", string(fields)),
			zap.String("error", err.Error()))
		if err == gorm.ErrRecordNotFound {
			return nil
		}
		return  errors.DataBaseError
	}
	return err
}

func (p *TProviderDevice) Update(db *gorm.DB, where map[string]interface{}, values map[string]interface{}) error {
	cond, val, err := whereBuild(where)
	if err != nil {
		return err
	}

	err = db.Table(p.TableName()).Table(p.TableName()).Where(cond, val...).Updates(values).Error
	if err != nil {
		fields, _ := json.Marshal(where)
		updateValues,_ := json.Marshal(values)
		logger.Error("mysql",
			zap.String("sql", "update t_provider_device"),
			zap.String("fields", string(fields)),
			zap.String("values", string(updateValues)),
			zap.String("error", err.Error()))
		return  errors.DataBaseError
	}
	return err
}

func (p *TProviderDevice) List(db *gorm.DB, where map[string]interface{}) (list []TProviderDevice, err error) {
	if len(where) > 0 {
		cond, val, err := whereBuild(where)
		if err != nil {
			return list, err
		}

		err = db.Table(p.TableName()).Where(cond, val...).Find(&list).Error
		if err != nil {
			fields, _ := json.Marshal(where)
			logger.Error("mysql",
				zap.String("sql", "select * t_provider_device"),
				zap.String("fields", string(fields)),
				zap.String("error", err.Error()))
			if err == gorm.ErrRecordNotFound {
				return nil, nil
			}
			return  nil, errors.DataBaseError
		}

		return list, err
	}
	err = db.Table(p.TableName()).Find(&list).Error
	if err != nil {
		logger.Error("mysql",
			zap.String("sql", "select * t_provider_device"),
			zap.String("error", err.Error()))
		if err == gorm.ErrRecordNotFound {
			return nil, nil
		}
		return  nil, errors.DataBaseError
	}

	return list, err
}

func (p *TProviderDevice) Count(db *gorm.DB, where map[string]interface{}) (int64, error) {
	if len(where) > 0 {
		cond, val, err := whereBuild(where)
		if err != nil {
			return 0, err
		}
		ret := int64(0)
		err = db.Table(p.TableName()).Where(cond, val...).Count(&ret).Error
		if err != nil {
			fields, _ := json.Marshal(where)
			logger.Error("mysql",
				zap.String("sql", "select count(1) t_provider_device"),
				zap.String("fields", string(fields)),
				zap.String("error", err.Error()))
			return  0, errors.DataBaseError
		}

		return ret, err
	}
	ret := int64(0)
	err := db.Table(p.TableName()).Count(&ret).Error
	if err != nil {
		logger.Error("mysql",
			zap.String("sql", "select count(1) t_provider_device"),
			zap.String("error", err.Error()))
		return  0, errors.DataBaseError
	}
	return ret, err
}

func (p *TProviderDevice) Del(db *gorm.DB, where map[string]interface{}) error {
	cond, val, err := whereBuild(where)
	if err != nil {
		return err
	}
	err = db.Table(p.TableName()).Where(cond, val...).Delete(p).Error
	if err != nil {
		fields, _ := json.Marshal(where)
		logger.Error("mysql",
			zap.String("sql", "delete from t_provider_device"),
			zap.String("fields", string(fields)),
			zap.String("error", err.Error()))
		return  errors.DataBaseError
	}
	return err
}
