// Copyright 2019  github.com. All rights reserved.
// Use of this source code is governed by  github.com.
package model

import (
	"fmt"
	"github.com/jinzhu/gorm"
	"go.uber.org/zap"
	"sync"
	"time"
	"github.com/jaryhe/gopkgs/logger"
	"smart-site-management/errors"
)

type TStaff struct {
	Id int64 `gorm:"column:id"`
	ProjectId int64 `gorm:"column:project_id"`
	WorkNo string `gorm:"column:work_no"`
	Name string `gorm:"column:name"`
	IdCert string `gorm:"column:id_cert"`
	Gender int64 `gorm:"column:gender"`
	StaffType int32 `gorm:"column:staff_type"`
	Birthday string `gorm:"column:birthday"`
	Address string `gorm:"column:address"`
	IdIssue string `gorm:"column:id_issue"`
	IdPeriod string `gorm:"column:id_period"`
	IdPhoto string `gorm:"column:id_photo"`
	Photo string `gorm:"column:photo"`
	InfPhoto string `gorm:"column:inf_photo"`
	Sn string `gorm:"column:sn"`
	Status int64 `gorm:"column:status"`
	Phone string `gorm:"column:phone"`
	WorkType int32 `gorm:"column:work_type"`
	LaborCompany string `gorm:"column:labor_company"`
	Group string `gorm:"column:group"`
	IsDelete bool `gorm:"column:is_delete"`
	CreatedAt time.Time `gorm:"column:created_at"`
	UpdatedAt time.Time `gorm:"column:updated_at"`
}

func (TStaff) TableName() string {
	return "db_smart_staff.t_staff"
}

type WorkType struct {
	Code int32
	WorkType string
}
var WorkTypeMap = map[int32]string{}
var workTypeMux sync.Mutex

type StaffType struct {
	StaffCode int32
	StaffType string
}
var StaffTypeMap = map[int32]string{}
var staffTypeMux sync.Mutex
func GetWorkTypeMap(db *gorm.DB) (map[int32]string, error) {
	workTypeMux.Lock()
	defer workTypeMux.Unlock()
	if len(WorkTypeMap) > 0 {
		return WorkTypeMap, nil
	}
	sql := "select work_type, code from db_smart_staff.t_work_type_item"
	array := []WorkType{}
	err := db.Raw(sql).Scan(&array).Error
	if err != nil {
		logger.Error("mysql",
			zap.String("sql", sql),
			zap.String("error", err.Error()))
		return nil, errors.DataBaseError
	}
	for _, v := range array {
		WorkTypeMap[v.Code] = v.WorkType
	}
	return WorkTypeMap, nil
}

func GetStaffTypeMap(db *gorm.DB) (map[int32]string, error) {
	staffTypeMux.Lock()
	defer staffTypeMux.Unlock()
	if len(StaffTypeMap) > 0 {
		return StaffTypeMap, nil
	}
	sql := "select staff_type, staff_code from db_smart_staff.t_staff_type_item"
	array := []StaffType{}
	err := db.Raw(sql).Scan(&array).Error
	if err != nil {
		logger.Error("mysql",
			zap.String("sql", sql),
			zap.String("error", err.Error()))
		return nil, errors.DataBaseError
	}
	for _, v := range array {
		StaffTypeMap[v.StaffCode] = v.StaffType
	}
	return StaffTypeMap, nil
}


/*
func (p *TStaff) List(db *gorm.DB, where map[string]interface{}, page int32, pageSize int32) (list []TStaff, err error) {
	offset := (page - 1) *pageSize
	if len(where) > 0 {
		cond, val, err := whereBuild(where)
		if err != nil {
			return list, err
		}

		err = db.Table(p.TableName()).Where(cond, val...).Limit(pageSize).Offset(offset).Find(&list).Error
		if err != nil {
			fields, _ := json.Marshal(where)
			logger.Error("mysql",
				zap.String("sql", "select * from t_device"),
				zap.String("fields", string(fields)),
				zap.String("error", err.Error()))
			if err == gorm.ErrRecordNotFound {
				return list, nil
			}
			return  list, errors.DataBaseError
		}

		return list, err
	}
	err = db.Table(p.TableName()).Limit(pageSize).Offset(offset).Find(&list).Error
	if err != nil {
		logger.Error("mysql",
			zap.String("sql", "select * from t_device"),
			zap.String("error", err.Error()))
		if err == gorm.ErrRecordNotFound {
			return list, nil
		}
		return  list, errors.DataBaseError
	}

	return list, err

}
*/

func (p *TStaff) List(db *gorm.DB, filter string, projectId int64, page int32, pageSize int32) (list []TStaff, total int64, err error) {
	offset := (page - 1) *pageSize
	where := ""
	whereArray := []string{}
	type ResultCount struct {
		Count int64
	}
	sql := fmt.Sprintf("select t1.* from db_smart_staff.t_staff as t1 left join db_smart_staff.t_staff_type_item as t2 on t1.staff_type=t2.staff_code left join db_smart_staff.t_work_type_item as t3 on t1.work_type=t3.code")
	countSql := fmt.Sprintf("select count(1) as count from db_smart_staff.t_staff as t1 left join db_smart_staff.t_staff_type_item as t2 on t1.staff_type=t2.staff_code left join db_smart_staff.t_work_type_item as t3 on t1.work_type=t3.code")
	if projectId > 0 {
		whereArray = append(whereArray, fmt.Sprintf("t1.project_id=%d", projectId))
	}
	if filter != "" {
		whereArray = append(whereArray, fmt.Sprintf("(t1.name like '%%%s%%' or t2.staff_type like '%%%s%%' or t3.work_type like '%%%s%%') ", filter, filter, filter))
	}
	for _, v := range whereArray {
		if where == "" {
			where = fmt.Sprintf(" where %s", v)
			continue
		}
		where = fmt.Sprintf("%s and %s", where, v)
	}
	sql = fmt.Sprintf("%s %s limit %d offset %d", sql, where, pageSize, offset)
	countSql = fmt.Sprintf("%s %s", countSql, where)

	array := []ResultCount{}
	ret := []TStaff{}

	err = db.Raw(countSql).Scan(&array).Error

	if err != nil {
		logger.Error("mysql",
			zap.String("sql", countSql),
			zap.String("error", err.Error()))
		return  nil, 0, errors.DataBaseError
	}
	if len(array) == 0 {
		return nil, 0, nil
	}
	if array[0].Count == 0 {
		return nil, 0, nil
	}

	err = db.Raw(sql).Scan(&ret).Error
	if err != nil {
		logger.Error("mysql",
			zap.String("sql", sql),
			zap.String("error", err.Error()))
		return nil, array[0].Count, errors.DataBaseError
	}
	return ret, array[0].Count, nil
}
