// Copyright 2019  github.com. All rights reserved.
// Use of this source code is governed by  github.com.
package model

import (
	"fmt"
	"github.com/jinzhu/gorm"
	"go.uber.org/zap"
	"smart-site-management/errors"
	"time"
	"github.com/jaryhe/gopkgs/logger"
	"encoding/json"
)

type Vssdevtbl struct {
	Id int64 `gorm:"column:ID" json:"id" form:"id"`
	Devpubid string `gorm:"column:DevPubID" json:"devpubid" form:"devpubid"`
	Devnickname string `gorm:"column:DevNickname" json:"devnickname" form:"devnickname"`
	Username string `gorm:"column:UserName" json:"username" form:"username"`
	Passwd string `gorm:"column:Passwd" json:"passwd" form:"passwd"`
	Corpid string `gorm:"column:CorpID" json:"corpid" form:"corpid"`
	Ip string `gorm:"column:IP" json:"ip" form:"ip"`
	Port int64 `gorm:"column:Port" json:"port" form:"port"`
	Alive int64 `gorm:"column:Alive" json:"alive" form:"alive"`
	Subcatalogtime int64 `gorm:"column:SubCatalogTime" json:"subcatalogtime" form:"subcatalogtime"`
	Devmode string `gorm:"column:DevMode" json:"devmode" form:"devmode"`
	Mediatransport string `gorm:"column:MediaTransport" json:"mediatransport" form:"mediatransport"`
	Dmarker int64 `gorm:"column:DMarker" json:"dmarker" form:"dmarker"`
	UpdateTime time.Time `gorm:"column:updatetime" json:"updatetime" form:"updatetime"`
}

func (Vssdevtbl) TableName() string {
	return "VSSDevTbl"
}


type CameraListRequest struct {
	IsUnused             bool     `protobuf:"varint,1,opt,name=is_unused,json=isUnused,proto3" json:"is_unused"`
	ProjectId            int64    `protobuf:"varint,2,opt,name=project_id,json=projectId,proto3" json:"project_id"`
	DeviceId             int64    `protobuf:"varint,3,opt,name=device_id,json=deviceId,proto3" json:"device_id"`
	IsAll                bool     `protobuf:"varint,4,opt,name=is_all,json=isAll,proto3" json:"is_all"`
	Page                 int32    `protobuf:"varint,5,opt,name=page,proto3" json:"page"`
	VssId                int64    `protobuf:"varint,6,opt,name=vss_id,json=vssId,proto3" json:"vss_id"`
}

type CameraListItem struct {
	VssSn                string   `json:"vss_sn"`
	VssName                 string   `json:"vss_name"`
	DeviceSn             string   `json:"device_sn"`
	DeviceId             int64    `json:"device_id"`
	ChannelId            int64    `json:"camera_id"`
	VssId                int64    `json:"vss_id"`
	ChannelNo string `json:"channel_id"`
	ChannelName string `json:"channel_name"`
}


func cameraListSql(req *CameraListRequest)(string, string, []interface{}) {
	sql := fmt.Sprintf("select t1.name as vss_name, t1.sn as vss_sn, t1.id as vss_id, t2.ID as channel_id,t2.ChanPubID as channel_no, t2.Nickname as channel_name, t3.id as device_id, t3.sn as device_sn from t_device as t1 left join VSSChannelTbl as t2 on t1.sn = t2.DevPubID left join t_device as t3 on t3.vss_channel_id=t2.ID and t3.device_code<>%d", DeviceTypeVedio)
	whereArray := []string{}
	where := ""
	countSql := fmt.Sprintf("select count(1) as count from t_device as t1 left join VSSChannelTbl as t2 on t1.sn = t2.DevPubID left join t_device as t3 on t3.vss_channel_id=t2.ID and t3.device_code<>%d", DeviceTypeVedio)
	args := []interface{}{}
	whereArray = append(whereArray, fmt.Sprintf("t1.device_code=%d", DeviceTypeVedio))
	if req.VssId > 0 {
		whereArray = append(whereArray, fmt.Sprintf("t1.id=%d", req.VssId))
	}
	if req.DeviceId > 0 {
		whereArray = append(whereArray, fmt.Sprintf("t3.id=%d", req.DeviceId))
	}
	if req.ProjectId > 0 {
		whereArray = append(whereArray, fmt.Sprintf("t1.project_id=%d", req.ProjectId))
	}
	if req.IsUnused {
		whereArray = append(whereArray, fmt.Sprintf("t3.id is null"))
	}

	for _, v := range whereArray {
		if where == "" {
			where = fmt.Sprintf(" where %s", v)
			continue
		}
		where = fmt.Sprintf("%s and %s", where, v)
	}

	offset := (req.Page - 1) *int32(req.Page)
	if req.IsAll {
		sql = fmt.Sprintf("%s %s", sql, where)
		countSql = fmt.Sprintf("%s %s", countSql, where)
	} else {
		sql = fmt.Sprintf("%s %s limit %d offset %d", sql, where, PageSize, offset)
		countSql = fmt.Sprintf("%s %s", countSql, where)
	}
	return sql, countSql, args

}

func CameraList(db *gorm.DB, req *CameraListRequest)([]CameraListItem, int64, error) {
	type ResultCount struct {
		Count int64
	}
	array := []ResultCount{}
	ret := []CameraListItem{}
	var err error
	sql, countSql, args := cameraListSql(req)

	err = db.Raw(countSql, args...).Scan(&array).Error

	if err != nil {
		fields, _ := json.Marshal(args)
		logger.Error("mysql",
			zap.String("sql", countSql),
			zap.String("fields", string(fields)),
			zap.String("error", err.Error()))

		return  nil, 0, errors.DataBaseError
	}

	if len(array) == 0 {
		return nil, 0, nil
	}
	if array[0].Count == 0 {
		return nil, 0, nil
	}

	err = db.Raw(sql, args...).Scan(&ret).Error
	if err != nil {
		fields, _ := json.Marshal(args)
		logger.Error("mysql",
			zap.String("sql", sql),
			zap.String("fields", string(fields)),
			zap.String("error", err.Error()))

		return  nil, array[0].Count, errors.DataBaseError
	}

	return ret, array[0].Count, nil
}
